"""Module providingFunction prepare dictionary for other scripts"""
from pathlib import Path

import rtoml
from markdown2 import markdown


class Formator:
    """read and convert ToMH files to dictionary"""
    def __init__(self) -> None:
        self.structure = {}
        self.base = {}
        self.base.update(rtoml.load(open("config.toml",encoding="utf-8")))
    def parse(self,input_str:str) -> dict:
        """parse ToMH into dictionary"""
        parsed_dict = {}
        input_split_list = input_str.replace("+++\n","").split("<!--break ")
        input_list = [n.split(" content-->") for n in input_split_list if n != ""]
        for do_list in input_list:
            if len(do_list) == 2:
                note_str, content_str = do_list
                note_list = [n.split(":") for n in note_str.split(" ") if n != ""]
                note_dict = {x:y for x,y in note_list}
                if note_dict["type"] == "header":
                    parsed_dict["header"] = rtoml.loads(content_str)
                elif note_dict["type"] == "content":
                    if note_dict["format"] == "md":
                        current_str = markdown(content_str,extras=["fenced-code-blocks","tables"])
                    else:
                        current_str = content_str
                    parsed_dict["content"] = current_str
                else:
                    type_dict = parsed_dict.get(note_dict["type"],{})
                    stored_str = type_dict.get(note_dict["title"],str())
                    combined_str = stored_str + content_str
                    simple_str = self.oneline(combined_str)
                    type_dict[note_dict["title"]] = simple_str
                    parsed_dict[note_dict["type"]] = type_dict
            else:
                print(F"WARN: {do_list}")
        return parsed_dict
    def load(self) -> None:
        """parse MoTH frame files"""
        target_list = []
        for folder_str in ["include_files","layout_files","page_files"]:
            target_list.extend(sorted(list(Path(folder_str).glob('*.*ml'))))
        for target_path in target_list:
            target_dict = self.parse(open(target_path,encoding="utf-8").read())
            for t_str in ["include","layout","format"]:
                if t_str in target_dict:
                    self.structure.update({F"{t_str}_{x}":y for x,y in target_dict[t_str].items()})
    def export(self):
        """export as TOML"""
        Path("mid_files").mkdir(exist_ok=True)
        with open("mid_files/structure.toml","w",encoding="utf-8") as toml_handle:
            rtoml.dump(self.structure,toml_handle)
    def oneline(self,input_str:str) -> str:
        """shrink into one line string"""
        return input_str.replace("\n","").replace("    ","")



"""Module providingFunction site generator"""
import argparse
from subprocess import call

from MaLTHA.convert import Convertor
from MaLTHA.database import Formator
from MaLTHA.generate import Generator

parser = argparse.ArgumentParser(description="generate webpage for github pages")
parser.add_argument("--skip", help="skip docs process", action="store_true")
parser.add_argument("--debug", help="debug mode", action="store_true")
args = parser.parse_args()

if args.skip:
    print("note: skip step 1 and step 2")
else:
    print("step 1: remove docs recursively")
    call(["rm","-r","docs"])
    print("step 2: copy static_files as docs recursively")
    call(["cp","-r","static_files","docs"])

print("step 3: remove mid_files recursively")
call(["rm","-r","mid_files"])

print("step 4: load include and layout")
Format = Formator()
Format.load()

print("step 5: convert files into JSONs")
if args.debug:
    Convert = Convertor(fmt=Format,bu_b=False)
else:
    Convert = Convertor(fmt=Format)
Convert.post()
Convert.category()
Convert.relate()
Convert.atom()
Convert.page()
Convert.output()

print("step 6: generate webpages from JSONs")
Generate = Generator(fmt=Format)
print("step 6-1: generate posts")
Generate.post()
print("step 6-2: generate pages")
Generate.page()
print("step 6-3: generate categories")
Generate.category()
print("step 6-4: complete pagination")
if Generate.base_info["paginate_format"] != "":
    Generate.pagination()



"""Module providingFunction convert JSON into HTML"""
import json
import math
from datetime import datetime, timedelta, timezone
from pathlib import Path

from MaLTHA.database import Formator


class Generator:
    """generator"""
    def __init__(self,fmt=Formator()) -> None:
        self.base_info = json.load(open("mid_files/base.json",encoding="utf-8"))
        self.categories_info = json.load(open("mid_files/categories.json",encoding="utf-8"))
        self.pages_dict = json.load(open("mid_files/page.json",encoding="utf-8"))
        self.posts_list = json.load(open("mid_files/post.json",encoding="utf-8"))
        self.fmt = fmt
        tz_element = timezone(timedelta(hours=8),name="UTC+8")
        self.cur_iso = datetime.now(tz=tz_element).isoformat()  # type: ignore
        print(F"Current time: {self.cur_iso}")
    def get(self,input_str:str) -> str:
        """grab dictionary from fmt.structure"""
        return self.fmt.structure[input_str]
    def template(self,input_str:str,input_dict:dict) -> str:
        """fill string into the dictionary from fmt.structure"""
        return self.fmt.structure[input_str].format(**input_dict)
    def post(self) -> None:
        """generate HTML from dictionary for posts"""
        for post_dict in self.posts_list:
            # post_dict = self.posts_list[0]
            base_dict = {}
            base_dict.update(self.fmt.structure)
            base_dict["layout_content"] = self.get("layout_post")
            base_dict.update(self.base_info)
            base_dict.update(post_dict)
            base_dict["canonical_url"] = post_dict["post_url"]
            base_dict["post_content"] = post_dict["content_full"]
            base_str = self.get("layout_default").format(**base_dict).format(**base_dict)
            base_str = base_str.replace("{{","{").replace("}}","}")
            short_canonical_str = post_dict["short_canonical"]
            bi_str = self.base_info["base_url"]
            url_list = ["docs"+n.replace(bi_str,"") for n in post_dict["post_urls"]]
            for path_str in url_list:
                Path(path_str).mkdir(parents=True,exist_ok=True)
                with open(F"{path_str}/index.html","w",encoding="utf-8") as target:
                    target.write(base_str)
    def page(self) -> None:
        """generate HTML from dictionary for pages"""
        keys_list = sorted(list(self.pages_dict.keys()))
        pages_list = [self.pages_dict[n] for n in keys_list]
        # page_dict = self.pages_dict["🔖 關於 About"]
        for page_dict in pages_list:
            if "page_content" in page_dict.keys():
                base_dict = {}
                base_dict.update(self.fmt.structure)
                base_dict["layout_content"] = self.get("layout_page")
                base_dict.update(self.base_info)
                base_dict.update(page_dict)
                base_dict["canonical_url"] = page_dict["page_url"]
                base_dict["current_iso8601"] = self.cur_iso
                if "base" in base_dict:
                    frame_str = base_dict["page_content"]
                else:
                    frame_str = self.get("layout_default")
                base_str = frame_str.format(**base_dict).format(**base_dict)
                short_canonical_str = page_dict["page_title"]
                if "frame" in page_dict.keys() or "layout" in page_dict.keys():
                    base_str = base_str.format(**base_dict)
                if "{" in base_str:
                    print(F"ERROR: need more formatting {short_canonical_str}")
                normal_str = self.template("format_pages_in_sidebar",page_dict)
                active_str = self.template("format_active_pages_in_sidebar",page_dict)
                if normal_str in base_str:
                    base_str = base_str.replace(normal_str,active_str)
                bi_str = self.base_info["base_url"]
                url_list = ["docs"+n.replace(bi_str,"") for n in page_dict["page_urls"]]
                base_str = base_str.replace("{{","{").replace("}}","}")
                for path_str in url_list:
                # path_str = url_list[0]
                    filename_str = str()
                    if path_str.split("/")[-1] == "":
                        Path(path_str).mkdir(parents=True,exist_ok=True)
                        filename_str = "index.html"
                    elif "." in path_str.split("/")[-1]:
                        Path(path_str).parent.mkdir(parents=True,exist_ok=True)
                        filename_str = str()
                    else:
                        Path(path_str).mkdir(parents=True,exist_ok=True)
                        filename_str = "/index.html"
                    with open(F"{path_str}{filename_str}","w",encoding="utf-8") as target:
                        target.write(base_str)
    def category(self):
        """generate HTML from dictionary for categories"""
        for category_str in sorted([n for n in self.categories_info.keys()]):
            category_dict = self.categories_info[category_str]
            base_dict = {}
            base_dict.update(self.fmt.structure)
            base_dict["layout_content"] = self.get("layout_category")
            base_dict.update(self.base_info)
            base_dict.update(category_dict)
            base_str = self.get("layout_default").format(**base_dict).format(**base_dict)
            base_str = base_str.replace("{{","{").replace("}}","}")
            short_canonical_str = category_dict["category_title"]
            if "{" in base_str:
                print(F"ERROR: need more formatting {short_canonical_str}")
            url_str = "docs"+category_dict["category_url"].replace(self.base_info["base_url"],"")
            Path(url_str).mkdir(parents=True,exist_ok=True)
            with open(F"{url_str}/index.html","w",encoding="utf-8") as target:
                target.write(base_str)
    def pagination(self):
        """generate HTML from dictionary for paginations"""
        posts_list = self.base_info["post_member_list"]
        paginate_format = self.base_info["paginate_format"]
        paginate_count = len(posts_list)
        paginate_number = self.base_info["paginate_number"]
        paginate_left = paginate_count%paginate_number
        paginate_ceil = math.ceil(paginate_count/paginate_number)
        paginate_dict = {}
        for pag_num in range(paginate_ceil):
            pag_cond = ((pag_num == paginate_ceil - 1) and (paginate_left > 0))
            paginate_length = paginate_left if pag_cond else paginate_number
            sn_int = pag_num*paginate_number
            en_int = (pag_num*paginate_number)+paginate_length
            paginate_dict[pag_num] = [posts_list[pos_num] for pos_num in range(sn_int,en_int)]
        for page_num, pag_list in paginate_dict.items():
            page_hold = paginate_format.format(num=page_num+1)
            post_dict = {}
            post_dict.update(self.fmt.structure)
            post_dict.update(self.base_info)
            post_dict["title"] = self.base_info["base_title"]
            post_dict["layout_content"] = self.get("layout_pagination")
            post_dict["pagination_content_list"] = "".join(pag_list)
            pob_str = "pagination_older_button"
            pnb_str = "pagination_newer_button"
            fpof_str = "format_pagination_older_froze"
            fpoa_str = "format_pagination_older_active"
            fpnf_str = "format_pagination_newer_froze"
            fpna_str = "format_pagination_newer_active"
            post_dict["canonical_url"] = self.base_info["base_url"]+page_hold
            if page_num == len(paginate_dict)-1:
                post_dict[pob_str] = self.get(fpof_str)
            else:
                old_hold = paginate_format.format(num=page_num+2)
                post_dict[pob_str] = self.get(fpoa_str).format(old_hold)
            url_list = []
            if page_num == 0:
                post_dict[pnb_str] = self.get(fpnf_str)
                url_list.append("docs/")
            elif page_num == 1:
                post_dict[pnb_str] = self.get(fpna_str).format("/")
            else:
                new_hold = paginate_format.format(num=page_num-1)
                post_dict[pnb_str] = self.get(fpna_str).format(new_hold)
            base_str = self.get("layout_default").format(**post_dict).format(**post_dict)
            base_str = base_str.replace("{{","{").replace("}}","}")
            url_list.append("docs"+page_hold)
            for path_str in url_list:
                Path(path_str).mkdir(parents=True,exist_ok=True)
                with open(F"{path_str}/index.html","w",encoding="utf-8") as target:
                    target.write(base_str)



"""Module providingFunction convert markdown/HTML/plain text into JSON"""
import json
from datetime import datetime
from pathlib import Path, PosixPath

from MaLTHA.database import Formator


class Convertor:
    """convert"""
    def __init__(self,bu_b=True,fmt=Formator()):
        self.fmt = fmt
        self.bs_d = {}
        self.bs_d.update(self.fmt.base)
        self.bu_s = self.bs_d["base_url"] if bu_b else str()
        self.pos_l = [] # posts_list
        self.cts_d = {} # categories_dict
        self.cts_c_l = [] # categories_content_list
        self.pgs_d = {} # pages_dict
    def template(self,input_str:str,input_dict:dict) -> str:
        """template"""
        return self.fmt.structure[input_str].format(**input_dict)
    def is_target(self,folder:PosixPath) -> bool:
        """for skip folder"""
        output = folder.is_dir()
        folder_name = str(folder.name)
        if folder_name[0] in {".","_"}:
            output = False
        if "_files" in folder_name:
            output = False
        if folder_name in {"docs","run"}:
            output = False
        return output
    def json(self,input_obj,target_path):
        """output as json"""
        with open(target_path,"w",encoding="utf-8") as target_handle:
            json.dump(input_obj,target_handle,indent=0)
    def post_ct(self,categories_list):
        """"Prepare category member dictionary from post"""
        category_parent_dict = {}
        category_content_list = []
        for category_str in categories_list:
            opengraph_description_str = self.bs_d["category_preview"].format(category_str)
            category_child_dict = {
                "title" : "·".join([category_str,self.bs_d["base_title"]]),
                "category_title" : category_str,
                "category_url" : self.bu_s+F"/category/{category_str}/",
                "canonical_url" : self.bu_s+F"/category/{category_str}/",
                "opengraph_description" : opengraph_description_str,
            }
            category_parent_dict[category_str] = category_child_dict
            tfc_str = self.template("format_categories_in_post",category_child_dict)
            category_content_list.append(tfc_str)
            #
            category_detail_dict = self.cts_d.get(category_str,{})
            category_detail_dict.update(category_child_dict)
            self.cts_d[category_str] = category_detail_dict
        return category_parent_dict,category_content_list
    def ct_member(self,post_dict):
        """Generate category member dictionary from post_ct()"""
        for category_str in post_dict["categories_dict"].keys():
            category_detail_dict = self.cts_d.get(category_str,{})
            category_member_dict = category_detail_dict.get("member",{})
            title_str = post_dict["post_title"]
            title_short_str = title_str[:15]+"..." if len(title_str) > 18 else title_str
            category_member_dict[post_dict["short_canonical"]] = {
                "member_title" : title_str,
                "member_short" : title_short_str,
                "member_url" : post_dict["post_url"],
                "member_date" : post_dict["date_show"],
            }
            category_detail_dict["member"] = category_member_dict
            self.cts_d[category_str] = category_detail_dict
    def path(self):
        """collect path for post"""
        post_paths = []
        for post_folder_posix_path in Path.cwd().iterdir():
            if self.is_target(post_folder_posix_path):
                post_paths.extend(sorted(list(post_folder_posix_path.glob('*.*'))))
        return post_paths
    def post(self):
        """convert func. for post"""
        for post_path in self.path():
            name_list = post_path.name.split(".")
            if len(name_list) > 1 and name_list[-1] in ["md","html"]:
                content_dict = self.fmt.parse(open(post_path,"r",encoding="utf-8").read())
                head_d = {}
                head_d.update(content_dict["header"])
                date_obj = datetime.fromisoformat(head_d["date"])
                dt_s = date_obj.strftime("%Y/%m/%d")
                # ct_p_d: category_parent_dict
                # ct_c_l: category_content_list
                ct_p_d,ct_c_l = self.post_ct(head_d["categories"])
                cts_s = "/".join(head_d["categories"])
                url_l = [self.bu_s+F"/{cts_s}/{dt_s}/{n}/" for n in head_d["short"]] # type: ignore
                url_l.extend([self.bu_s+F"/{cts_s}/{n}/" for n in head_d["short"]]) # type: ignore
                url_l.extend([self.bu_s+F"/{dt_s}/{n}/" for n in head_d["short"]]) # type: ignore
                url_l.extend([self.bu_s+F"/{n}/" for n in head_d["short"]]) # type: ignore
                url_l.append(self.bu_s+"/post/"+head_d["short"][0]+"/")
                post_url_str = self.bu_s+F"/{dt_s}/"+head_d["short"][0]+"/"
                content_split_list = content_dict["content"].split(self.bs_d["separator_preview"])
                if len(content_split_list) == 1:
                    more_word_str = self.bs_d["read_original"]
                else:
                    more_word_str = self.bs_d["read_more"]
                po_d = {} # po_d
                po_d.update(head_d)
                po_d.update({
                    "title" : " · ".join([head_d["title"],self.bs_d["base_title"]]),
                    "short_list" : head_d["short"],
                    "short_canonical" : head_d["short"][0],
                    "categories_dict" : ct_p_d,
                    "date_iso" : head_d["date"],
                    "date_show" : date_obj.strftime("%a, %b %-d, %Y"),
                    "date_822" : date_obj.strftime("%a, %d %b %Y %T %z"),
                    "date_8601" : date_obj.isoformat(),
                    "post_title" : head_d["title"],
                    "post_urls" : url_l,
                    "post_url" : post_url_str,
                    "post_categories": "".join(ct_c_l),
                    "content_full" : content_dict["content"],
                    "content_preview" : self.fmt.oneline(content_split_list[0]),
                    "more_element" : F"<a href=\"{post_url_str}\">{more_word_str}</a>",
                })
                self.ct_member(po_d)
                self.pos_l.append(po_d)
        self.check_post()
    def check_post(self):
        """Check post if duplicate or not"""
        pos_l = [post_dict["short_canonical"] for post_dict in self.pos_l]
        dup_l = [po for po in set(pos_l) if pos_l.count(po) > 1]
        if len(dup_l) > 0:
            print(F"ERROR: duplicate id; {dup_l}")
    def category(self):
        """prepare categories"""
        for category_str in sorted([n for n in self.cts_d.keys()]):
            detl_d = self.cts_d[category_str]
            content_list = []
            section_list = []
            for member_dict in detl_d["member"].values():
                content_list.append(self.template("format_member_in_category_content",member_dict))
                section_list.append(self.template("format_member_in_category_section",member_dict))
            detl_d["category_content"] = "".join(content_list)
            detl_d["category_section"] = "".join(section_list)
            self.cts_d[category_str] = detl_d
            self.cts_c_l.append(self.template("format_categories_by_section",detl_d))
        self.bs_d["categories_content_list"] = "".join(self.cts_c_l)
    def relate(self):
        """prepare related section"""
        frm_s = "format_related_member"
        pos_d = {po_d["short_canonical"]:i for i,po_d in enumerate(self.pos_l)}
        for pos, po_d in enumerate(self.pos_l):
            rlt_d = {} # related_dict
            for category_str in po_d["categories_dict"].keys():
                category_member_dict = self.cts_d[category_str]["member"]
                rlt_d.update({x:self.template(frm_s,y) for x,y in category_member_dict.items()})
            rlt_od_d = {pos_d[n]:n for n in rlt_d if n != po_d["short_canonical"]}
            rlt_od_l = [rlt_d[rlt_od_d[n]] for n in sorted(list(rlt_od_d.keys()),reverse=True)]
            related_list = rlt_od_l[:3] if len(rlt_od_l) > 3 else rlt_od_l
            related_str = "".join(related_list)
            # po_d["related_order_list"] = rlt_od_l
            frf_s = "format_related_frame"
            if frf_s in self.fmt.structure.keys():
                po_d["related_content"] = self.template(frf_s,{"related_posts_list":related_str})
            self.pos_l[pos] = po_d
    def atom(self):
        """prepare atom/preview section"""
        post_member_list = []
        post_atom_list = []
        rv_od_l = [self.pos_l[pos] for pos in range(len(self.pos_l)-1, -1, -1)]
        for post_dict in rv_od_l:
            if post_dict["content_full"] == post_dict["content_preview"]:
                post_member_list.append(self.template("format_post_container_full",post_dict))
            else:
                post_member_list.append(self.template("format_post_container_preview",post_dict))
            atom_dict = {}
            atom_dict.update(post_dict)
            atom_bool = self.bs_d["base_url"] in post_dict["post_url"]
            atom_dict["base_url"] = str() if atom_bool else self.bs_d["base_url"]
            post_atom_list.append(self.template("format_atom_post",atom_dict))
        self.bs_d["post_member_list"] = post_member_list
        self.bs_d["post_content_list"] = "".join(post_member_list)
        self.bs_d["post_atom_list"] = post_atom_list
        self.bs_d["atom_content_list"] = "".join(post_atom_list)
    def page(self):
        """convert func. for page"""
        for page_path in sorted(list(Path("page_files").glob('*.*'))):
            content_dict = self.fmt.parse(open(page_path,encoding="utf-8").read())
            head_d = {}
            head_d.update(content_dict["header"])
            url_l = [self.bu_s+F"{n}" for n in head_d["path"]]
            url_str = url_l[0]
            if head_d.get("skip","") != "content":
                url_l.append(self.bu_s+F"/pages{url_str}")
            page_detail_dict = {
                "title" : " · ".join([head_d["title"],self.bs_d["base_title"]]),
                "page_title" : head_d["title"],
                "page_urls" : url_l,
                "page_url" : url_str,
            }
            page_dict = {}
            page_dict.update(head_d)
            page_dict.update(page_detail_dict)
            type_list = ["base","layout","frame"]
            type_in_head_l = [n for n in type_list if n in head_d]
            frm_d = content_dict.get("frame",{})
            type_in_content_list = [n for n in type_in_head_l if head_d[n] in frm_d]
            if head_d.get("skip","") != "content":
                if "content" in content_dict.keys():
                    page_dict["page_content"] = content_dict["content"]
                elif len(type_in_content_list) > 0:
                    type_str = type_in_content_list[0]
                    page_dict["page_content"] = content_dict["frame"][head_d[type_str]]
                else:
                    print("ERROR: can't get content from "+head_d["title"])
            if "layout" in head_d:
                if head_d["layout"] in content_dict["frame"].keys():
                    page_dict["layout_content"] = content_dict["frame"][head_d["layout"]]
                else:
                    print("ERROR: can't get layout from "+head_d["title"])
            page_dict.update({n:head_d[n] for n in type_in_head_l})
            if head_d.get("skip","") == "list":
                page_dict["skip_list"] = ""
            if head_d["title"] in self.pgs_d:
                print("ERROR: duplicate canonical id ["+head_d["title"]+"]")
            else:
                self.pgs_d[head_d["title"]] = page_dict
        side_page_list = [n for n in self.pgs_d.values() if "skip_list" not in n.keys()]
        page_content_list = [self.template("format_pages_in_sidebar",n) for n in side_page_list]
        self.bs_d["page_content_list"] = "".join(page_content_list)
    def output(self):
        """output JSON files"""
        Path("mid_files").mkdir(exist_ok=True)
        self.json(self.bs_d,"mid_files/base.json")
        self.json(self.pos_l,"mid_files/post.json")
        pos_d = {po_d["short_canonical"]:i for i,po_d in enumerate(self.pos_l)}
        self.json(pos_d,"mid_files/post_pos.json")
        self.json(self.cts_d,"mid_files/categories.json")
        self.json(self.pgs_d,"mid_files/page.json")



import setuptools

setuptools.setup(
    name = "MaLTHA",
    version = "0.3.2.1",
    author = "David Soh",
    author_email = "ln@trth.nl",
    description = "MaLTHA works as a static site generator that use Markdown format with TOML/HTML hybrid annotation.",
    long_description = open('README.md').read(),
    long_description_content_type = 'text/markdown',
    url = "https://github.com/SotongDJ/MaLTHA",
    packages = ["MaLTHA"],
    keywords = [],
    classifiers = [
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "rtoml",
        "markdown2",
        "Pygments",
    ],
)
